@@ -28,17 +28,16 @@ import traceback
 
 from pylons import request
 from pylons import tmpl_context as c, url
-from pylons.controllers.util import redirect
+from pylons.controllers.util import redirect, abort
 
-from webhelpers.paginate import Page
-
-from rhodecode.lib.base import BaseController, render
 from rhodecode.model.db import Notification
-
 from rhodecode.model.notification import NotificationModel
-from rhodecode.lib.auth import LoginRequired, NotAnonymous
-from rhodecode.lib import helpers as h
 from rhodecode.model.meta import Session
+from rhodecode.lib.auth import LoginRequired, NotAnonymous
+from rhodecode.lib.base import BaseController, render
+from rhodecode.lib import helpers as h
+from rhodecode.lib.helpers import Page
+from rhodecode.lib.utils2 import safe_int
 
 
 log = logging.getLogger(__name__)
@@ -62,7 +61,8 @@ class NotificationsController(BaseController):
         c.user = self.rhodecode_user
         notif = NotificationModel().get_for_user(self.rhodecode_user.user_id,
                                             filter_=request.GET.getall('type'))
-        p = int(request.params.get('page', 1))
+
+        p = safe_int(request.GET.get('page', 1), 1)
         c.notifications = Page(notif, page=p, items_per_page=10)
         c.pull_request_type = Notification.TYPE_PULL_REQUEST
         c.comment_type = [Notification.TYPE_CHANGESET_COMMENT,
@@ -108,14 +108,14 @@ class NotificationsController(BaseController):
         # url('notification', notification_id=ID)
         try:
             no = Notification.get(notification_id)
-            owner = lambda: (no.notifications_to_users.user.user_id
-                             == c.rhodecode_user.user_id)
+            owner = all(un.user.user_id == c.rhodecode_user.user_id
+                        for un in no.notifications_to_users)
             if h.HasPermissionAny('hg.admin')() or owner:
                     NotificationModel().mark_read(c.rhodecode_user.user_id, no)
                     Session().commit()
                     return 'ok'
         except Exception:
-            Session.rollback()
+            Session().rollback()
             log.error(traceback.format_exc())
         return 'fail'
 
@@ -130,14 +130,14 @@ class NotificationsController(BaseController):
 
         try:
             no = Notification.get(notification_id)
-            owner = lambda: (no.notifications_to_users.user.user_id
-                             == c.rhodecode_user.user_id)
+            owner = all(un.user.user_id == c.rhodecode_user.user_id
+                        for un in no.notifications_to_users)
             if h.HasPermissionAny('hg.admin')() or owner:
                     NotificationModel().delete(c.rhodecode_user.user_id, no)
                     Session().commit()
                     return 'ok'
         except Exception:
-            Session.rollback()
+            Session().rollback()
             log.error(traceback.format_exc())
         return 'fail'
 
@@ -147,8 +147,9 @@ class NotificationsController(BaseController):
         c.user = self.rhodecode_user
         no = Notification.get(notification_id)
 
-        owner = lambda: (no.notifications_to_users.user.user_id
-                         == c.user.user_id)
+        owner = any(un.user.user_id == c.rhodecode_user.user_id
+                    for un in no.notifications_to_users)
+
         if no and (h.HasPermissionAny('hg.admin', 'repository.admin')() or owner):
             unotification = NotificationModel()\
                             .get_user_notification(c.user.user_id, no)
@@ -156,14 +157,14 @@ class NotificationsController(BaseController):
             # if this association to user is not valid, we don't want to show
             # this message
             if unotification:
-                if unotification.read is False:
+                if not unotification.read:
                     unotification.mark_as_read()
                     Session().commit()
                 c.notification = no
 
                 return render('admin/notifications/show_notification.html')
 
-        return redirect(url('notifications'))
+        return abort(403)
 
     def edit(self, notification_id, format='html'):
         """GET /_admin/notifications/id/edit: Form to edit an existing item"""
