@@ -30,63 +30,221 @@ from webob.exc import HTTPNotFound, HTTPForbidden
 from collections import defaultdict
 from itertools import groupby
 
-from pylons import request, response, session, tmpl_context as c, url
-from pylons.controllers.util import abort, redirect
+from pylons import request, tmpl_context as c, url
+from pylons.controllers.util import redirect
 from pylons.i18n.translation import _
-from pylons.decorators import jsonify
 
 from rhodecode.lib.compat import json
 from rhodecode.lib.base import BaseRepoController, render
 from rhodecode.lib.auth import LoginRequired, HasRepoPermissionAnyDecorator,\
     NotAnonymous
+from rhodecode.lib.helpers import Page
 from rhodecode.lib import helpers as h
 from rhodecode.lib import diffs
-from rhodecode.lib.utils import action_logger
-from rhodecode.model.db import User, PullRequest, ChangesetStatus,\
-    ChangesetComment
+from rhodecode.lib.utils import action_logger, jsonify
+from rhodecode.lib.vcs.utils import safe_str
+from rhodecode.lib.vcs.exceptions import EmptyRepositoryError
+from rhodecode.lib.diffs import LimitedDiffContainer
+from rhodecode.model.db import  PullRequest, ChangesetStatus, ChangesetComment
 from rhodecode.model.pull_request import PullRequestModel
 from rhodecode.model.meta import Session
 from rhodecode.model.repo import RepoModel
 from rhodecode.model.comment import ChangesetCommentsModel
 from rhodecode.model.changeset_status import ChangesetStatusModel
 from rhodecode.model.forms import PullRequestForm
+from rhodecode.lib.utils2 import safe_int
 
 log = logging.getLogger(__name__)
 
 
 class PullrequestsController(BaseRepoController):
 
-    @LoginRequired()
-    @HasRepoPermissionAnyDecorator('repository.read', 'repository.write',
-                                   'repository.admin')
     def __before__(self):
         super(PullrequestsController, self).__before__()
         repo_model = RepoModel()
         c.users_array = repo_model.get_users_js()
         c.users_groups_array = repo_model.get_users_groups_js()
 
-    def _get_repo_refs(self, repo):
-        hist_l = []
+    def _get_repo_refs(self, repo, rev=None, branch=None, branch_rev=None):
+        """return a structure with repo's interesting changesets, suitable for
+        the selectors in pullrequest.html
 
-        branches_group = ([('branch:%s:%s' % (k, v), k) for
-                         k, v in repo.branches.iteritems()], _("Branches"))
-        bookmarks_group = ([('book:%s:%s' % (k, v), k) for
-                         k, v in repo.bookmarks.iteritems()], _("Bookmarks"))
-        tags_group = ([('tag:%s:%s' % (k, v), k) for
-                         k, v in repo.tags.iteritems()], _("Tags"))
+        rev: a revision that must be in the list somehow and selected by default
+        branch: a branch that must be in the list and selected by default - even if closed
+        branch_rev: a revision of which peers should be preferred and available."""
+        # list named branches that has been merged to this named branch - it should probably merge back
+        peers = []
 
-        hist_l.append(bookmarks_group)
-        hist_l.append(branches_group)
-        hist_l.append(tags_group)
+        if rev:
+            rev = safe_str(rev)
 
-        return hist_l
+        if branch:
+            branch = safe_str(branch)
 
+        if branch_rev:
+            branch_rev = safe_str(branch_rev)
+            # not restricting to merge() would also get branch point and be better
+            # (especially because it would get the branch point) ... but is currently too expensive
+            otherbranches = {}
+            for i in repo._repo.revs(
+                "sort(parents(branch(id(%s)) and merge()) - branch(id(%s)))",
+                branch_rev, branch_rev):
+                cs = repo.get_changeset(i)
+                otherbranches[cs.branch] = cs.raw_id
+            for abranch, node in otherbranches.iteritems():
+                selected = 'branch:%s:%s' % (abranch, node)
+                peers.append((selected, abranch))
+
+        selected = None
+
+        branches = []
+        for abranch, branchrev in repo.branches.iteritems():
+            n = 'branch:%s:%s' % (abranch, branchrev)
+            branches.append((n, abranch))
+            if rev == branchrev:
+                selected = n
+            if branch == abranch:
+                selected = n
+                branch = None
+        if branch: # branch not in list - it is probably closed
+            revs = repo._repo.revs('max(branch(%s))', branch)
+            if revs:
+                cs = repo.get_changeset(revs[0])
+                selected = 'branch:%s:%s' % (branch, cs.raw_id)
+                branches.append((selected, branch))
+
+        bookmarks = []
+        for bookmark, bookmarkrev in repo.bookmarks.iteritems():
+            n = 'book:%s:%s' % (bookmark, bookmarkrev)
+            bookmarks.append((n, bookmark))
+            if rev == bookmarkrev:
+                selected = n
+
+        tags = []
+        for tag, tagrev in repo.tags.iteritems():
+            n = 'tag:%s:%s' % (tag, tagrev)
+            tags.append((n, tag))
+            if rev == tagrev and tag != 'tip': # tip is not a real tag - and its branch is better
+                selected = n
+
+        # prio 1: rev was selected as existing entry above
+
+        # prio 2: create special entry for rev; rev _must_ be used
+        specials = []
+        if rev and selected is None:
+            selected = 'rev:%s:%s' % (rev, rev)
+            specials = [(selected, '%s: %s' % (_("Changeset"), rev[:12]))]
+
+        # prio 3: most recent peer branch
+        if peers and not selected:
+            selected = peers[0][0][0]
+
+        # prio 4: tip revision
+        if not selected:
+            selected = 'tag:tip:%s' % repo.tags['tip']
+
+        groups = [(specials, _("Special")),
+                  (peers, _("Peer branches")),
+                  (bookmarks, _("Bookmarks")),
+                  (branches, _("Branches")),
+                  (tags, _("Tags")),
+                  ]
+        return [g for g in groups if g[0]], selected
+
+    def _get_is_allowed_change_status(self, pull_request):
+        owner = self.rhodecode_user.user_id == pull_request.user_id
+        reviewer = self.rhodecode_user.user_id in [x.user_id for x in
+                                                   pull_request.reviewers]
+        return (self.rhodecode_user.admin or owner or reviewer)
+
+    def _load_compare_data(self, pull_request, enable_comments=True):
+        """
+        Load context data needed for generating compare diff
+
+        :param pull_request:
+        """
+        org_repo = pull_request.org_repo
+        (org_ref_type,
+         org_ref_name,
+         org_ref_rev) = pull_request.org_ref.split(':')
+
+        other_repo = org_repo
+        (other_ref_type,
+         other_ref_name,
+         other_ref_rev) = pull_request.other_ref.split(':')
+
+        # despite opening revisions for bookmarks/branches/tags, we always
+        # convert this to rev to prevent changes after bookmark or branch change
+        org_ref = ('rev', org_ref_rev)
+        other_ref = ('rev', other_ref_rev)
+
+        c.org_repo = org_repo
+        c.other_repo = other_repo
+
+        c.fulldiff = fulldiff = request.GET.get('fulldiff')
+
+        c.cs_ranges = [org_repo.get_changeset(x) for x in pull_request.revisions]
+
+        c.statuses = org_repo.statuses([x.raw_id for x in c.cs_ranges])
+
+        c.org_ref = org_ref[1]
+        c.org_ref_type = org_ref[0]
+        c.other_ref = other_ref[1]
+        c.other_ref_type = other_ref[0]
+
+        diff_limit = self.cut_off_limit if not fulldiff else None
+
+        # we swap org/other ref since we run a simple diff on one repo
+        log.debug('running diff between %s and %s in %s'
+                  % (other_ref, org_ref, org_repo.scm_instance.path))
+        txtdiff = org_repo.scm_instance.get_diff(rev1=safe_str(other_ref[1]), rev2=safe_str(org_ref[1]))
+
+        diff_processor = diffs.DiffProcessor(txtdiff or '', format='gitdiff',
+                                             diff_limit=diff_limit)
+        _parsed = diff_processor.prepare()
+
+        c.limited_diff = False
+        if isinstance(_parsed, LimitedDiffContainer):
+            c.limited_diff = True
+
+        c.files = []
+        c.changes = {}
+        c.lines_added = 0
+        c.lines_deleted = 0
+
+        for f in _parsed:
+            st = f['stats']
+            c.lines_added += st['added']
+            c.lines_deleted += st['deleted']
+            fid = h.FID('', f['filename'])
+            c.files.append([fid, f['operation'], f['filename'], f['stats']])
+            htmldiff = diff_processor.as_html(enable_comments=enable_comments,
+                                              parsed_lines=[f])
+            c.changes[fid] = [f['operation'], f['filename'], htmldiff]
+
+    @LoginRequired()
+    @HasRepoPermissionAnyDecorator('repository.read', 'repository.write',
+                                   'repository.admin')
     def show_all(self, repo_name):
-        c.pull_requests = PullRequestModel().get_all(repo_name)
+        c.from_ = request.GET.get('from_') or ''
+        c.closed = request.GET.get('closed') or ''
+        c.pull_requests = PullRequestModel().get_all(repo_name, from_=c.from_, closed=c.closed)
         c.repo_name = repo_name
+        p = safe_int(request.GET.get('page', 1), 1)
+
+        c.pullrequests_pager = Page(c.pull_requests, page=p, items_per_page=10)
+
+        c.pullrequest_data = render('/pullrequests/pullrequest_data.html')
+
+        if request.environ.get('HTTP_X_PARTIAL_XHR'):
+            return c.pullrequest_data
+
         return render('/pullrequests/pullrequest_show_all.html')
 
+    @LoginRequired()
     @NotAnonymous()
+    @HasRepoPermissionAnyDecorator('repository.read', 'repository.write',
+                                   'repository.admin')
     def index(self):
         org_repo = c.rhodecode_db_repo
 
@@ -94,62 +252,69 @@ class PullrequestsController(BaseRepoController):
             log.error('Review not available for GIT REPOS')
             raise HTTPNotFound
 
+        try:
+            org_repo.scm_instance.get_changeset()
+        except EmptyRepositoryError, e:
+            h.flash(h.literal(_('There are no changesets yet')),
+                    category='warning')
+            redirect(url('summary_home', repo_name=org_repo.repo_name))
+
+        org_rev = request.GET.get('rev_end')
+        # rev_start is not directly useful - its parent could however be used
+        # as default for other and thus give a simple compare view
+        #other_rev = request.POST.get('rev_start')
+        branch = request.GET.get('branch')
+
+        c.org_repos = []
+        c.org_repos.append((org_repo.repo_name, org_repo.repo_name))
+        c.default_org_repo = org_repo.repo_name
+        c.org_refs, c.default_org_ref = self._get_repo_refs(org_repo.scm_instance, rev=org_rev, branch=branch)
+
+        c.other_repos = []
         other_repos_info = {}
 
-        c.org_refs = self._get_repo_refs(c.rhodecode_repo)
-        c.org_repos = []
-        c.other_repos = []
-        c.org_repos.append((org_repo.repo_name, '%s/%s' % (
-                                org_repo.user.username, c.repo_name))
-                           )
+        def add_other_repo(repo, branch_rev=None):
+            if repo.repo_name in other_repos_info: # shouldn't happen
+                return
+            c.other_repos.append((repo.repo_name, repo.repo_name))
+            other_refs, selected_other_ref = self._get_repo_refs(repo.scm_instance, branch_rev=branch_rev)
+            other_repos_info[repo.repo_name] = {
+                'user': dict(user_id=repo.user.user_id,
+                             username=repo.user.username,
+                             firstname=repo.user.firstname,
+                             lastname=repo.user.lastname,
+                             gravatar_link=h.gravatar_url(repo.user.email, 14)),
+                'description': repo.description.split('\n', 1)[0],
+                'revs': h.select('other_ref', selected_other_ref, other_refs, class_='refs')
+            }
 
-        # add org repo to other so we can open pull request agains itself
-        c.other_repos.extend(c.org_repos)
+        # add org repo to other so we can open pull request against peer branches on itself
+        add_other_repo(org_repo, branch_rev=org_rev)
+        c.default_other_repo = org_repo.repo_name
 
-        c.default_pull_request = org_repo.repo_name
-        c.default_revs = self._get_repo_refs(org_repo.scm_instance)
-        #add orginal repo
-        other_repos_info[org_repo.repo_name] = {
-            'gravatar': h.gravatar_url(org_repo.user.email, 24),
-            'description': org_repo.description,
-            'revs': h.select('other_ref', '', c.default_revs, class_='refs')
-        }
-
-        #gather forks and add to this list
+        # gather forks and add to this list ... even though it is rare to
+        # request forks to pull from their parent
         for fork in org_repo.forks:
-            c.other_repos.append((fork.repo_name, '%s/%s' % (
-                                    fork.user.username, fork.repo_name))
-                                 )
-            other_repos_info[fork.repo_name] = {
-                'gravatar': h.gravatar_url(fork.user.email, 24),
-                'description': fork.description,
-                'revs': h.select('other_ref', '',
-                                 self._get_repo_refs(fork.scm_instance),
-                                 class_='refs')
-            }
-        #add parents of this fork also
-        if org_repo.parent:
-            c.default_pull_request = org_repo.parent.repo_name
-            c.other_repos.append((org_repo.parent.repo_name, '%s/%s' % (
-                                        org_repo.parent.user.username,
-                                        org_repo.parent.repo_name))
-                                     )
-            other_repos_info[org_repo.parent.repo_name] = {
-                'gravatar': h.gravatar_url(org_repo.parent.user.email, 24),
-                'description': org_repo.parent.description,
-                'revs': h.select('other_ref', '',
-                                 self._get_repo_refs(org_repo.parent.scm_instance),
-                                 class_='refs')
-            }
+            add_other_repo(fork)
 
+        # add parents of this fork also, but only if it's not empty
+        if org_repo.parent and org_repo.parent.scm_instance.revisions:
+            add_other_repo(org_repo.parent)
+            c.default_other_repo = org_repo.parent.repo_name
+
+        c.default_other_repo_info = other_repos_info[c.default_other_repo]
         c.other_repos_info = json.dumps(other_repos_info)
-        c.review_members = [org_repo.user]
+
         return render('/pullrequests/pullrequest.html')
 
+    @LoginRequired()
     @NotAnonymous()
+    @HasRepoPermissionAnyDecorator('repository.read', 'repository.write',
+                                   'repository.admin')
     def create(self, repo_name):
+        repo = RepoModel()._get_repo(repo_name)
         try:
-            _form = PullRequestForm()().to_python(request.POST)
+            _form = PullRequestForm(repo.repo_id)().to_python(request.POST)
         except formencode.Invalid, errors:
             log.error(traceback.format_exc())
             if errors.error_dict.get('revisions'):
@@ -157,21 +322,22 @@ class PullrequestsController(BaseRepoController):
             elif errors.error_dict.get('pullrequest_title'):
                 msg = _('Pull request requires a title with min. 3 chars')
             else:
-                msg = _('error during creation of pull request')
+                msg = _('Error creating pull request: %s') % errors.msg
 
             h.flash(msg, 'error')
-            return redirect(url('pullrequest_home', repo_name=repo_name))
+            return redirect(url('pullrequest_home', repo_name=repo_name)) ## would rather just go back to form ...
 
         org_repo = _form['org_repo']
-        org_ref = _form['org_ref']
+        org_ref = _form['org_ref'] # will end with merge_rev but have symbolic name
         other_repo = _form['other_repo']
-        other_ref = _form['other_ref']
-        revisions = _form['revisions']
+        other_ref = 'rev:ancestor:%s' % _form['ancestor_rev'] # could be calculated from other_ref ...
+        revisions = [x for x in reversed(_form['revisions'])]
         reviewers = _form['review_members']
 
         title = _form['pullrequest_title']
+        if not title:
+            title = '%s#%s to %s' % (org_repo, org_ref.split(':', 2)[1], other_repo)
         description = _form['pullrequest_desc']
-
         try:
             pull_request = PullRequestModel().create(
                 self.rhodecode_user.user_id, org_repo, org_ref, other_repo,
@@ -189,7 +355,10 @@ class PullrequestsController(BaseRepoController):
         return redirect(url('pullrequest_show', repo_name=other_repo,
                             pull_request_id=pull_request.pull_request_id))
 
+    @LoginRequired()
     @NotAnonymous()
+    @HasRepoPermissionAnyDecorator('repository.read', 'repository.write',
+                                   'repository.admin')
     @jsonify
     def update(self, repo_name, pull_request_id):
         pull_request = PullRequest.get_or_404(pull_request_id)
@@ -202,11 +371,14 @@ class PullrequestsController(BaseRepoController):
                        request.POST.get('reviewers_ids', '').split(',')))
 
             PullRequestModel().update_reviewers(pull_request_id, reviewers_ids)
-            Session.commit()
+            Session().commit()
             return True
         raise HTTPForbidden()
 
+    @LoginRequired()
     @NotAnonymous()
+    @HasRepoPermissionAnyDecorator('repository.read', 'repository.write',
+                                   'repository.admin')
     @jsonify
     def delete(self, repo_name, pull_request_id):
         pull_request = PullRequest.get_or_404(pull_request_id)
@@ -216,68 +388,18 @@ class PullrequestsController(BaseRepoController):
             Session().commit()
             h.flash(_('Successfully deleted pull request'),
                     category='success')
-            return redirect(url('admin_settings_my_account'))
+            return redirect(url('admin_settings_my_account', anchor='pullrequests'))
         raise HTTPForbidden()
 
-    def _load_compare_data(self, pull_request, enable_comments=True):
-        """
-        Load context data needed for generating compare diff
-
-        :param pull_request:
-        :type pull_request:
-        """
-
-        org_repo = pull_request.org_repo
-        (org_ref_type,
-         org_ref_name,
-         org_ref_rev) = pull_request.org_ref.split(':')
-
-        other_repo = pull_request.other_repo
-        (other_ref_type,
-         other_ref_name,
-         other_ref_rev) = pull_request.other_ref.split(':')
-
-        # despite opening revisions for bookmarks/branches/tags, we always
-        # convert this to rev to prevent changes after book or branch change
-        org_ref = ('rev', org_ref_rev)
-        other_ref = ('rev', other_ref_rev)
-
-        c.org_repo = org_repo
-        c.other_repo = other_repo
-
-        c.cs_ranges, discovery_data = PullRequestModel().get_compare_data(
-                                       org_repo, org_ref, other_repo, other_ref
-                                      )
-
-        c.statuses = c.rhodecode_db_repo.statuses([x.raw_id for x in
-                                                   c.cs_ranges])
-        # defines that we need hidden inputs with changesets
-        c.as_form = request.GET.get('as_form', False)
-
-        c.org_ref = org_ref[1]
-        c.other_ref = other_ref[1]
-        # diff needs to have swapped org with other to generate proper diff
-        _diff = diffs.differ(other_repo, other_ref, org_repo, org_ref,
-                             discovery_data)
-        diff_processor = diffs.DiffProcessor(_diff, format='gitdiff')
-        _parsed = diff_processor.prepare()
-
-        c.files = []
-        c.changes = {}
-
-        for f in _parsed:
-            fid = h.FID('', f['filename'])
-            c.files.append([fid, f['operation'], f['filename'], f['stats']])
-            diff = diff_processor.as_html(enable_comments=enable_comments,
-                                          diff_lines=[f])
-            c.changes[fid] = [f['operation'], f['filename'], diff]
-
+    @LoginRequired()
+    @HasRepoPermissionAnyDecorator('repository.read', 'repository.write',
+                                   'repository.admin')
     def show(self, repo_name, pull_request_id):
         repo_model = RepoModel()
         c.users_array = repo_model.get_users_js()
         c.users_groups_array = repo_model.get_users_groups_js()
         c.pull_request = PullRequest.get_or_404(pull_request_id)
-
+        c.allowed_to_change_status = self._get_is_allowed_change_status(c.pull_request)
         cc_model = ChangesetCommentsModel()
         cs_model = ChangesetStatusModel()
         _cs_statuses = cs_model.get_statuses(c.pull_request.org_repo,
@@ -322,15 +444,20 @@ class PullrequestsController(BaseRepoController):
         c.comments = cc_model.get_comments(c.rhodecode_db_repo.repo_id,
                                            pull_request=pull_request_id)
 
-        # changeset(pull-request) status
+        # (badly named) pull-request status calculation based on reviewer votes
         c.current_changeset_status = cs_model.calculate_status(
-                                        c.pull_request_reviewers
-                                     )
+                                        c.pull_request_reviewers,
+                                         )
         c.changeset_statuses = ChangesetStatus.STATUSES
-        c.target_repo = c.pull_request.org_repo.repo_name
+
+        c.as_form = False
+        c.ancestor = None # there is one - but right here we don't know which
         return render('/pullrequests/pullrequest_show.html')
 
+    @LoginRequired()
     @NotAnonymous()
+    @HasRepoPermissionAnyDecorator('repository.read', 'repository.write',
+                                   'repository.admin')
     @jsonify
     def comment(self, repo_name, pull_request_id):
         pull_request = PullRequest.get_or_404(pull_request_id)
@@ -339,36 +466,54 @@ class PullrequestsController(BaseRepoController):
 
         status = request.POST.get('changeset_status')
         change_status = request.POST.get('change_changeset_status')
+        text = request.POST.get('text')
+        close_pr = request.POST.get('save_close')
 
+        allowed_to_change_status = self._get_is_allowed_change_status(pull_request)
+        if status and change_status and allowed_to_change_status:
+            _def = (_('Status change -> %s')
+                            % ChangesetStatus.get_status_lbl(status))
+            if close_pr:
+                _def = _('Closing with') + ' ' + _def
+            text = text or _def
         comm = ChangesetCommentsModel().create(
-            text=request.POST.get('text'),
+            text=text,
             repo=c.rhodecode_db_repo.repo_id,
             user=c.rhodecode_user.user_id,
             pull_request=pull_request_id,
             f_path=request.POST.get('f_path'),
             line_no=request.POST.get('line'),
             status_change=(ChangesetStatus.get_status_lbl(status)
-                           if status and change_status else None)
+                           if status and change_status
+                           and allowed_to_change_status else None),
+            closing_pr=close_pr
         )
 
-        # get status if set !
-        if status and change_status:
-            ChangesetStatusModel().set_status(
-                c.rhodecode_db_repo.repo_id,
-                status,
-                c.rhodecode_user.user_id,
-                comm,
-                pull_request=pull_request_id
-            )
         action_logger(self.rhodecode_user,
                       'user_commented_pull_request:%s' % pull_request_id,
                       c.rhodecode_db_repo, self.ip_addr, self.sa)
 
-        if request.POST.get('save_close'):
-            PullRequestModel().close_pull_request(pull_request_id)
-            action_logger(self.rhodecode_user,
-                      'user_closed_pull_request:%s' % pull_request_id,
-                      c.rhodecode_db_repo, self.ip_addr, self.sa)
+        if allowed_to_change_status:
+            # get status if set !
+            if status and change_status:
+                ChangesetStatusModel().set_status(
+                    c.rhodecode_db_repo.repo_id,
+                    status,
+                    c.rhodecode_user.user_id,
+                    comm,
+                    pull_request=pull_request_id
+                )
+
+            if close_pr:
+                if status in ['rejected', 'approved']:
+                    PullRequestModel().close_pull_request(pull_request_id)
+                    action_logger(self.rhodecode_user,
+                              'user_closed_pull_request:%s' % pull_request_id,
+                              c.rhodecode_db_repo, self.ip_addr, self.sa)
+                else:
+                    h.flash(_('Closing pull request on other statuses than '
+                              'rejected or approved forbidden'),
+                            category='warning')
 
         Session().commit()
 
@@ -387,7 +532,10 @@ class PullrequestsController(BaseRepoController):
 
         return data
 
+    @LoginRequired()
     @NotAnonymous()
+    @HasRepoPermissionAnyDecorator('repository.read', 'repository.write',
+                                   'repository.admin')
     @jsonify
     def delete_comment(self, repo_name, comment_id):
         co = ChangesetComment.get(comment_id)
@@ -395,7 +543,7 @@ class PullrequestsController(BaseRepoController):
             #don't allow deleting comments on closed pull request
             raise HTTPForbidden()
 
-        owner = lambda: co.author.user_id == c.rhodecode_user.user_id
+        owner = co.author.user_id == c.rhodecode_user.user_id
         if h.HasPermissionAny('hg.admin', 'repository.admin')() or owner:
             ChangesetCommentsModel().delete(comment=co)
             Session().commit()
