@@ -24,34 +24,34 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 import logging
 import traceback
-
+import urllib
 from pylons.i18n.translation import _
 from pylons import request, config, tmpl_context as c
 
-from rhodecode.lib.auth import LoginRequired
-from rhodecode.lib.base import BaseController, render
-from rhodecode.lib.indexers import CHGSETS_SCHEMA, SCHEMA, CHGSET_IDX_NAME, \
-    IDX_NAME, WhooshResultWrapper
-
-from webhelpers.paginate import Page
-from webhelpers.util import update_params
-
 from whoosh.index import open_dir, EmptyIndexError
 from whoosh.qparser import QueryParser, QueryParserError
 from whoosh.query import Phrase, Wildcard, Term, Prefix
+from webhelpers.util import update_params
+
+from rhodecode.lib.auth import LoginRequired
+from rhodecode.lib.base import BaseRepoController, render
+from rhodecode.lib.indexers import CHGSETS_SCHEMA, SCHEMA, CHGSET_IDX_NAME, \
+    IDX_NAME, WhooshResultWrapper
 from rhodecode.model.repo import RepoModel
+from rhodecode.lib.utils2 import safe_str, safe_int
+from rhodecode.lib.helpers import Page
 
 log = logging.getLogger(__name__)
 
 
-class SearchController(BaseController):
+class SearchController(BaseRepoController):
 
-    @LoginRequired()
     def __before__(self):
         super(SearchController, self).__before__()
 
-    def index(self, search_repo=None):
-        c.repo_name = search_repo
+    @LoginRequired()
+    def index(self, repo_name=None):
+        c.repo_name = repo_name
         c.formated_results = []
         c.runtime = ''
         c.cur_query = request.GET.get('q', None)
@@ -82,7 +82,7 @@ class SearchController(BaseController):
             log.debug(cur_query)
 
         if c.cur_query:
-            p = int(request.params.get('page', 1))
+            p = safe_int(request.GET.get('page', 1), 1)
             highlight_items = set()
             try:
                 idx = open_dir(config['app_conf']['index_dir'],
@@ -115,8 +115,9 @@ class SearchController(BaseController):
                     )
 
                     def url_generator(**kw):
+                        q = urllib.quote(safe_str(c.cur_query))
                         return update_params("?q=%s&type=%s" \
-                                           % (c.cur_query, c.cur_type), **kw)
+                        % (q, safe_str(c.cur_type)), **kw)
                     repo_location = RepoModel().repos_path
                     c.formated_results = Page(
                         WhooshResultWrapper(search_type, searcher, matcher,
