@@ -12,13 +12,16 @@ def get_current_revision(quiet=False):
     try:
         from rhodecode.lib.vcs import get_repo
         from rhodecode.lib.vcs.utils.helpers import get_scm
-        repopath = os.path.join(os.path.dirname(__file__), '..', '..')
+        repopath = os.path.abspath(os.path.join(os.path.dirname(__file__),
+                                                '..', '..'))
         scm = get_scm(repopath)[0]
         repo = get_repo(path=repopath, alias=scm)
-        tip = repo.get_changeset()
-        return (tip.revision, tip.short_id)
+        wk_dir = repo.workdir
+        cur_rev = wk_dir.get_changeset()
+        return (cur_rev.revision, cur_rev.short_id)
     except Exception, err:
         if not quiet:
-            print ("Cannot retrieve rhodecode's revision. Original error "
-                   "was: %s" % err)
+            print ("WARNING: Cannot retrieve rhodecode's revision. "
+                   "disregard this if you don't know what that means. "
+                   "Original error was: %s" % err)
         return None
