@@ -25,13 +25,17 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import os
-from rhodecode import __platform__, PLATFORM_WIN, __py_version__
+from rhodecode import __py_version__, is_windows
 
 #==============================================================================
 # json
 #==============================================================================
 from rhodecode.lib.ext_json import json
 
+if __py_version__ >= (2, 7):
+    import unittest
+else:
+    import unittest2 as unittest
 
 #==============================================================================
 # izip_longest
@@ -365,7 +369,7 @@ from sqlalchemy.util import OrderedSet
 #==============================================================================
 # kill FUNCTIONS
 #==============================================================================
-if __platform__ in PLATFORM_WIN:
+if is_windows:
     import ctypes
 
     def kill(pid, sig):
@@ -415,6 +419,15 @@ else:
     # in py2.6 bytes is a synonim for str
     _bytes = str
 
+if __py_version__ >= (2, 6):
+    _bytearray = bytearray
+else:
+    import array
+    # no idea if this is correct but all integration tests are passing
+    # i think we never use bytearray anyway
+    _bytearray = array
+
+
 #==============================================================================
 # deque
 #==============================================================================
@@ -539,56 +552,3 @@ else:
             memo[id(self)] = result
             result.__init__(deepcopy(tuple(self), memo))
             return result
-
-
-#==============================================================================
-# threading.Event
-#==============================================================================
-
-if __py_version__ >= (2, 6):
-    from threading import Event, Thread
-else:
-    from threading import _Verbose, Condition, Lock, Thread
-
-    def Event(*args, **kwargs):
-        return _Event(*args, **kwargs)
-
-    class _Event(_Verbose):
-
-        # After Tim Peters' event class (without is_posted())
-
-        def __init__(self, verbose=None):
-            _Verbose.__init__(self, verbose)
-            self.__cond = Condition(Lock())
-            self.__flag = False
-
-        def isSet(self):
-            return self.__flag
-
-        is_set = isSet
-
-        def set(self):
-            self.__cond.acquire()
-            try:
-                self.__flag = True
-                self.__cond.notify_all()
-            finally:
-                self.__cond.release()
-
-        def clear(self):
-            self.__cond.acquire()
-            try:
-                self.__flag = False
-            finally:
-                self.__cond.release()
-
-        def wait(self, timeout=None):
-            self.__cond.acquire()
-            try:
-                if not self.__flag:
-                    self.__cond.wait(timeout)
-            finally:
-                self.__cond.release()
-
-
-
