@@ -2,18 +2,20 @@ import re
 from itertools import chain
 from dulwich import objects
 from subprocess import Popen, PIPE
+
 from rhodecode.lib.vcs.conf import settings
-from rhodecode.lib.vcs.exceptions import RepositoryError
-from rhodecode.lib.vcs.exceptions import ChangesetError
-from rhodecode.lib.vcs.exceptions import NodeDoesNotExistError
-from rhodecode.lib.vcs.exceptions import VCSError
-from rhodecode.lib.vcs.exceptions import ChangesetDoesNotExistError
-from rhodecode.lib.vcs.exceptions import ImproperArchiveTypeError
 from rhodecode.lib.vcs.backends.base import BaseChangeset, EmptyChangeset
-from rhodecode.lib.vcs.nodes import FileNode, DirNode, NodeKind, RootNode, \
-    RemovedFileNode, SubModuleNode
-from rhodecode.lib.vcs.utils import safe_unicode
-from rhodecode.lib.vcs.utils import date_fromtimestamp
+from rhodecode.lib.vcs.exceptions import (
+    RepositoryError, ChangesetError, NodeDoesNotExistError, VCSError,
+    ChangesetDoesNotExistError, ImproperArchiveTypeError
+)
+from rhodecode.lib.vcs.nodes import (
+    FileNode, DirNode, NodeKind, RootNode, RemovedFileNode, SubModuleNode,
+    ChangedFileNodesGenerator, AddedFileNodesGenerator, RemovedFileNodesGenerator
+)
+from rhodecode.lib.vcs.utils import (
+    safe_unicode, safe_str, safe_int, date_fromtimestamp
+)
 from rhodecode.lib.vcs.utils.lazy import LazyProperty
 
 
@@ -27,7 +29,7 @@ class GitChangeset(BaseChangeset):
         self.repository = repository
 
         try:
-            commit = self.repository._repo.get_object(revision)
+            commit = self.repository._repo[revision]
             if isinstance(commit, objects.Tag):
                 revision = commit.object[1]
                 commit = self.repository._repo.get_object(commit.object[1])
@@ -37,22 +39,27 @@ class GitChangeset(BaseChangeset):
         self.id = self.raw_id
         self.short_id = self.raw_id[:12]
         self._commit = commit
-
         self._tree_id = commit.tree
-        self._commiter_property = 'committer'
+        self._committer_property = 'committer'
+        self._author_property = 'author'
         self._date_property = 'commit_time'
         self._date_tz_property = 'commit_timezone'
         self.revision = repository.revisions.index(revision)
 
-        self.message = safe_unicode(commit.message)
-        #self.branch = None
-        self.tags = []
         self.nodes = {}
         self._paths = {}
 
+    @LazyProperty
+    def message(self):
+        return safe_unicode(self._commit.message)
+
+    @LazyProperty
+    def committer(self):
+        return safe_unicode(getattr(self._commit, self._committer_property))
+
     @LazyProperty
     def author(self):
-        return safe_unicode(getattr(self._commit, self._commiter_property))
+        return safe_unicode(getattr(self._commit, self._author_property))
 
     @LazyProperty
     def date(self):
@@ -70,6 +77,14 @@ class GitChangeset(BaseChangeset):
         """
         return self.changed, self.added, self.removed
 
+    @LazyProperty
+    def tags(self):
+        _tags = []
+        for tname, tsha in self.repository.tags.iteritems():
+            if tsha == self.raw_id:
+                _tags.append(tname)
+        return _tags
+
     @LazyProperty
     def branch(self):
 
@@ -89,7 +104,7 @@ class GitChangeset(BaseChangeset):
         return path
 
     def _get_id_for_path(self, path):
-
+        path = safe_str(path)
         # FIXME: Please, spare a couple of minutes and make those codes cleaner;
         if not path in self._paths:
             path = path.strip('/')
@@ -138,8 +153,8 @@ class GitChangeset(BaseChangeset):
                     self._stat_modes[name] = stat
             if not path in self._paths:
                 raise NodeDoesNotExistError("There is no file nor directory "
-                    "at the given path %r at revision %r"
-                    % (path, self.short_id))
+                    "at the given path '%s' at revision %s"
+                    % (path, safe_str(self.short_id)))
         return self._paths[path]
 
     def _get_kind(self, path):
@@ -149,6 +164,13 @@ class GitChangeset(BaseChangeset):
         elif isinstance(obj, objects.Tree):
             return NodeKind.DIR
 
+    def _get_filectx(self, path):
+        path = self._fix_path(path)
+        if self._get_kind(path) != NodeKind.FILE:
+            raise ChangesetError("File does not exist for revision %s at "
+                " '%s'" % (self.raw_id, path))
+        return path
+
     def _get_file_nodes(self):
         return chain(*(t[2] for t in self.walk()))
 
@@ -160,6 +182,22 @@ class GitChangeset(BaseChangeset):
         return [self.repository.get_changeset(parent)
                 for parent in self._commit.parents]
 
+    @LazyProperty
+    def children(self):
+        """
+        Returns list of children changesets.
+        """
+        rev_filter = _git_path = settings.GIT_REV_FILTER
+        so, se = self.repository.run_git_command(
+            "rev-list %s --children | grep '^%s'" % (rev_filter, self.raw_id)
+        )
+
+        children = []
+        for l in so.splitlines():
+            childs = l.split(' ')[1:]
+            children.extend(childs)
+        return [self.repository.get_changeset(cs) for cs in children]
+
     def next(self, branch=None):
 
         if branch and self.branch != branch:
@@ -216,6 +254,7 @@ class GitChangeset(BaseChangeset):
         Returns stat mode of the file at the given ``path``.
         """
         # ensure path is traversed
+        path = safe_str(path)
         self._get_id_for_path(path)
         return self._stat_modes[path]
 
@@ -239,10 +278,9 @@ class GitChangeset(BaseChangeset):
         """
         Returns last commit of the file at the given ``path``.
         """
-        node = self.get_node(path)
-        return node.history[0]
+        return self.get_file_history(path, limit=1)[0]
 
-    def get_file_history(self, path):
+    def get_file_history(self, path, limit=None):
         """
         Returns history of file as reversed list of ``Changeset`` objects for
         which file at given ``path`` has been modified.
@@ -251,16 +289,39 @@ class GitChangeset(BaseChangeset):
         which is generally not good. Should be replaced with algorithm
         iterating commits.
         """
-        cmd = 'log --pretty="format: %%H" -s -p %s -- "%s"' % (
-                  self.id, path
-               )
+        self._get_filectx(path)
+        cs_id = safe_str(self.id)
+        f_path = safe_str(path)
+
+        if limit:
+            cmd = 'log -n %s --pretty="format: %%H" -s %s -- "%s"' % (
+                      safe_int(limit, 0), cs_id, f_path)
+
+        else:
+            cmd = 'log --pretty="format: %%H" -s %s -- "%s"' % (
+                      cs_id, f_path)
         so, se = self.repository.run_git_command(cmd)
         ids = re.findall(r'[0-9a-fA-F]{40}', so)
-        return [self.repository.get_changeset(id) for id in ids]
+        return [self.repository.get_changeset(sha) for sha in ids]
+
+    def get_file_history_2(self, path):
+        """
+        Returns history of file as reversed list of ``Changeset`` objects for
+        which file at given ``path`` has been modified.
+
+        """
+        self._get_filectx(path)
+        from dulwich.walk import Walker
+        include = [self.id]
+        walker = Walker(self.repository._repo.object_store, include,
+                        paths=[path], max_entries=1)
+        return [self.repository.get_changeset(sha)
+                for sha in (x.commit.id for x in walker)]
 
     def get_file_annotate(self, path):
         """
-        Returns a list of three element tuples with lineno,changeset and line
+        Returns a generator of four element tuples with
+            lineno, sha, changeset lazy loader and line
 
         TODO: This function now uses os underlying 'git' command which is
         generally not good. Should be replaced with algorithm iterating
@@ -272,12 +333,10 @@ class GitChangeset(BaseChangeset):
         # -r sha ==> blames for the given revision
         so, se = self.repository.run_git_command(cmd)
 
-        annotate = []
         for i, blame_line in enumerate(so.split('\n')[:-1]):
             ln_no = i + 1
-            id, line = re.split(r' ', blame_line, 1)
-            annotate.append((ln_no, self.repository.get_changeset(id), line))
-        return annotate
+            sha, line = re.split(r' ', blame_line, 1)
+            yield (ln_no, sha, lambda: self.repository.get_changeset(sha), line)
 
     def fill_archive(self, stream=None, kind='tgz', prefix=None,
                      subrepos=False):
@@ -312,8 +371,9 @@ class GitChangeset(BaseChangeset):
             frmt = 'zip'
         else:
             frmt = 'tar'
-        cmd = 'git archive --format=%s --prefix=%s/ %s' % (frmt, prefix,
-            self.raw_id)
+        _git_path = settings.GIT_EXECUTABLE_PATH
+        cmd = '%s archive --format=%s --prefix=%s/ %s' % (_git_path,
+                                                frmt, prefix, self.raw_id)
         if kind == 'tgz':
             cmd += ' | gzip -9'
         elif kind == 'tbz2':
@@ -335,8 +395,8 @@ class GitChangeset(BaseChangeset):
 
     def get_nodes(self, path):
         if self._get_kind(path) != NodeKind.DIR:
-            raise ChangesetError("Directory does not exist for revision %r at "
-                " %r" % (self.revision, path))
+            raise ChangesetError("Directory does not exist for revision %s at "
+                " '%s'" % (self.revision, path))
         path = self._fix_path(path)
         id = self._get_id_for_path(path)
         tree = self.repository._repo[id]
@@ -399,7 +459,7 @@ class GitChangeset(BaseChangeset):
                     node._blob = obj
                 else:
                     raise NodeDoesNotExistError("There is no file nor directory "
-                        "at the given path %r at revision %r"
+                        "at the given path '%s' at revision %s"
                         % (path, self.short_id))
             # cache node
             self.nodes[path] = node
@@ -408,10 +468,10 @@ class GitChangeset(BaseChangeset):
     @LazyProperty
     def affected_files(self):
         """
-        Get's a fast accessible file changes for given changeset
+        Gets a fast accessible file changes for given changeset
         """
-        a, m, d = self._changes_cache
-        return list(a.union(m).union(d))
+        added, modified, deleted = self._changes_cache
+        return list(added.union(modified).union(deleted))
 
     @LazyProperty
     def _diff_name_status(self):
@@ -454,11 +514,11 @@ class GitChangeset(BaseChangeset):
 
         :param status: one of: *added*, *modified* or *deleted*
         """
-        a, m, d = self._changes_cache
+        added, modified, deleted = self._changes_cache
         return sorted({
-            'added': list(a),
-            'modified': list(m),
-            'deleted': list(d)}[status]
+            'added': list(added),
+            'modified': list(modified),
+            'deleted': list(deleted)}[status]
         )
 
     @LazyProperty
@@ -468,7 +528,8 @@ class GitChangeset(BaseChangeset):
         """
         if not self.parents:
             return list(self._get_file_nodes())
-        return [self.get_node(path) for path in self._get_paths_for_status('added')]
+        return AddedFileNodesGenerator([n for n in
+                                self._get_paths_for_status('added')], self)
 
     @LazyProperty
     def changed(self):
@@ -477,7 +538,8 @@ class GitChangeset(BaseChangeset):
         """
         if not self.parents:
             return []
-        return [self.get_node(path) for path in self._get_paths_for_status('modified')]
+        return ChangedFileNodesGenerator([n for n in
+                                self._get_paths_for_status('modified')], self)
 
     @LazyProperty
     def removed(self):
@@ -486,4 +548,5 @@ class GitChangeset(BaseChangeset):
         """
         if not self.parents:
             return []
-        return [RemovedFileNode(path) for path in self._get_paths_for_status('deleted')]
+        return RemovedFileNodesGenerator([n for n in
+                                self._get_paths_for_status('deleted')], self)
