@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 """
-    vcs.backends.git
-    ~~~~~~~~~~~~~~~~
+    vcs.backends.git.repository
+    ~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-    Git backend implementation.
+    Git repository implementation.
 
     :created_on: Apr 8, 2010
     :copyright: (c) 2010-2011 by Marcin Kuzminski, Lukasz Balcerzak.
@@ -12,33 +12,38 @@
 import os
 import re
 import time
-import posixpath
-import logging
-import traceback
 import urllib
 import urllib2
+import logging
+import posixpath
+import string
+
+from dulwich.objects import Tag
 from dulwich.repo import Repo, NotGitRepository
-#from dulwich.config import ConfigFile
-from string import Template
-from subprocess import Popen, PIPE
-from rhodecode.lib.vcs.backends.base import BaseRepository
-from rhodecode.lib.vcs.exceptions import BranchDoesNotExistError
-from rhodecode.lib.vcs.exceptions import ChangesetDoesNotExistError
-from rhodecode.lib.vcs.exceptions import EmptyRepositoryError
-from rhodecode.lib.vcs.exceptions import RepositoryError
-from rhodecode.lib.vcs.exceptions import TagAlreadyExistError
-from rhodecode.lib.vcs.exceptions import TagDoesNotExistError
+
+from rhodecode.lib.vcs import subprocessio
+from rhodecode.lib.vcs.backends.base import BaseRepository, CollectionGenerator
+from rhodecode.lib.vcs.conf import settings
+
+from rhodecode.lib.vcs.exceptions import (
+    BranchDoesNotExistError, ChangesetDoesNotExistError, EmptyRepositoryError,
+    RepositoryError, TagAlreadyExistError, TagDoesNotExistError
+)
 from rhodecode.lib.vcs.utils import safe_unicode, makedate, date_fromtimestamp
 from rhodecode.lib.vcs.utils.lazy import LazyProperty
 from rhodecode.lib.vcs.utils.ordered_dict import OrderedDict
-from rhodecode.lib.vcs.utils.paths import abspath
-from rhodecode.lib.vcs.utils.paths import get_user_home
-from .workdir import GitWorkdir
-from .changeset import GitChangeset
-from .inmemory import GitInMemoryChangeset
-from .config import ConfigFile
-from rhodecode.lib import subprocessio
+from rhodecode.lib.vcs.utils.paths import abspath, get_user_home
 
+from rhodecode.lib.vcs.utils.hgcompat import (
+    hg_url, httpbasicauthhandler, httpdigestauthhandler
+)
+
+from .changeset import GitChangeset
+from .config import ConfigFile
+from .inmemory import GitInMemoryChangeset
+from .workdir import GitWorkdir
+
+SHA_PATTERN = re.compile(r'^[[0-9a-fA-F]{12}|[0-9a-fA-F]{40}]$')
 
 log = logging.getLogger(__name__)
 
@@ -54,27 +59,39 @@ class GitRepository(BaseRepository):
                  update_after_clone=False, bare=False):
 
         self.path = abspath(repo_path)
-        self._repo = self._get_repo(create, src_url, update_after_clone, bare)
-        #temporary set that to now at later we will move it to constructor
-        baseui = None
-        if baseui is None:
-            from mercurial.ui import ui
-            baseui = ui()
-        # patch the instance of GitRepo with an "FAKE" ui object to add
-        # compatibility layer with Mercurial
-        setattr(self._repo, 'ui', baseui)
+        repo = self._get_repo(create, src_url, update_after_clone, bare)
+        self.bare = repo.bare
+
+    @property
+    def _config_files(self):
+        return [
+            self.bare and abspath(self.path, 'config')
+                      or abspath(self.path, '.git', 'config'),
+             abspath(get_user_home(), '.gitconfig'),
+         ]
+
+    @property
+    def _repo(self):
+        return Repo(self.path)
+
+    @property
+    def head(self):
+        try:
+            return self._repo.head()
+        except KeyError:
+            return None
+
+    @property
+    def _empty(self):
+        """
+        Checks if repository is empty ie. without any changesets
+        """
 
         try:
-            self.head = self._repo.head()
-        except KeyError:
-            self.head = None
-
-        self._config_files = [
-            bare and abspath(self.path, 'config') or abspath(self.path, '.git',
-                'config'),
-            abspath(get_user_home(), '.gitconfig'),
-        ]
-        self.bare = self._repo.bare
+            self.revisions[0]
+        except (KeyError, IndexError):
+            return True
+        return False
 
     @LazyProperty
     def revisions(self):
@@ -84,20 +101,27 @@ class GitRepository(BaseRepository):
         """
         return self._get_all_revisions()
 
-    def run_git_command(self, cmd):
+    @classmethod
+    def _run_git_command(cls, cmd, **opts):
         """
         Runs given ``cmd`` as git command and returns tuple
-        (returncode, stdout, stderr).
-
-        .. note::
-           This method exists only until log/blame functionality is implemented
-           at Dulwich (see https://bugs.launchpad.net/bugs/645142). Parsing
-           os command's output is road to hell...
+        (stdout, stderr).
 
         :param cmd: git command to be executed
+        :param opts: env options to pass into Subprocess command
         """
 
-        _copts = ['-c', 'core.quotepath=false', ]
+        if '_bare' in opts:
+            _copts = []
+            del opts['_bare']
+        else:
+            _copts = ['-c', 'core.quotepath=false', ]
+        safe_call = False
+        if '_safe' in opts:
+            #no exc on failure
+            del opts['_safe']
+            safe_call = True
+
         _str_cmd = False
         if isinstance(cmd, basestring):
             cmd = [cmd]
@@ -109,24 +133,35 @@ class GitRepository(BaseRepository):
             del gitenv['GIT_DIR']
         gitenv['GIT_CONFIG_NOGLOBAL'] = '1'
 
-        cmd = ['git'] + _copts + cmd
+        _git_path = settings.GIT_EXECUTABLE_PATH
+        cmd = [_git_path] + _copts + cmd
         if _str_cmd:
             cmd = ' '.join(cmd)
+
         try:
-            opts = dict(
+            _opts = dict(
                 env=gitenv,
-                shell=False,
+                shell=True,
             )
-            if os.path.isdir(self.path):
-                opts['cwd'] = self.path
-            p = subprocessio.SubprocessIOChunker(cmd, **opts)
+            _opts.update(opts)
+            p = subprocessio.SubprocessIOChunker(cmd, **_opts)
         except (EnvironmentError, OSError), err:
-            log.error(traceback.format_exc())
-            raise RepositoryError("Couldn't run git command (%s).\n"
-                                  "Original error was:%s" % (cmd, err))
+            tb_err = ("Couldn't run git command (%s).\n"
+                      "Original error was:%s\n" % (cmd, err))
+            log.error(tb_err)
+            if safe_call:
+                return '', err
+            else:
+                raise RepositoryError(tb_err)
 
         return ''.join(p.output), ''.join(p.error)
 
+    def run_git_command(self, cmd):
+        opts = {}
+        if os.path.isdir(self.path):
+            opts['cwd'] = self.path
+        return self._run_git_command(cmd, **opts)
+
     @classmethod
     def _check_url(cls, url):
         """
@@ -137,11 +172,6 @@ class GitRepository(BaseRepository):
 
         On failures it'll raise urllib2.HTTPError
         """
-        from mercurial.util import url as Url
-
-        # those authnadlers are patched for python 2.6.5 bug an
-        # infinit looping when given invalid resources
-        from mercurial.url import httpbasicauthhandler, httpdigestauthhandler
 
         # check first if it's not an local url
         if os.path.isdir(url) or url.startswith('file:'):
@@ -151,7 +181,7 @@ class GitRepository(BaseRepository):
             url = url[url.find('+') + 1:]
 
         handlers = []
-        test_uri, authinfo = Url(url).authinfo()
+        test_uri, authinfo = hg_url(url).authinfo()
         if not test_uri.endswith('info/refs'):
             test_uri = test_uri.rstrip('/') + '/info/refs'
         if authinfo:
@@ -178,7 +208,7 @@ class GitRepository(BaseRepository):
             raise urllib2.URLError("[%s] %s" % (url, e))
 
     def _get_repo(self, create, src_url=None, update_after_clone=False,
-            bare=False):
+                  bare=False):
         if create and os.path.exists(self.path):
             raise RepositoryError("Location already exist")
         if src_url and not create:
@@ -196,7 +226,7 @@ class GitRepository(BaseRepository):
                 else:
                     return Repo.init(self.path)
             else:
-                return Repo(self.path)
+                return self._repo
         except (NotGitRepository, OSError), err:
             raise RepositoryError(err)
 
@@ -208,7 +238,9 @@ class GitRepository(BaseRepository):
             self._repo.head()
         except KeyError:
             return []
-        cmd = 'rev-list --all --reverse --date-order'
+
+        rev_filter = _git_path = settings.GIT_REV_FILTER
+        cmd = 'rev-list %s --reverse --date-order' % (rev_filter)
         try:
             so, se = self.run_git_command(cmd)
         except RepositoryError:
@@ -227,42 +259,42 @@ class GitRepository(BaseRepository):
         For git backend we always return integer here. This way we ensure
         that changset's revision attribute would become integer.
         """
-        pattern = re.compile(r'^[[0-9a-fA-F]{12}|[0-9a-fA-F]{40}]$')
-        is_bstr = lambda o: isinstance(o, (str, unicode))
+
         is_null = lambda o: len(o) == revision.count('0')
 
-        if len(self.revisions) == 0:
+        if self._empty:
             raise EmptyRepositoryError("There are no changesets yet")
 
         if revision in (None, '', 'tip', 'HEAD', 'head', -1):
-            revision = self.revisions[-1]
+            return self.revisions[-1]
 
-        if ((is_bstr(revision) and revision.isdigit() and len(revision) < 12)
+        is_bstr = isinstance(revision, (str, unicode))
+        if ((is_bstr and revision.isdigit() and len(revision) < 12)
             or isinstance(revision, int) or is_null(revision)):
             try:
                 revision = self.revisions[int(revision)]
-            except:
-                raise ChangesetDoesNotExistError("Revision %r does not exist "
-                    "for this repository %s" % (revision, self))
+            except Exception:
+                raise ChangesetDoesNotExistError("Revision %s does not exist "
+                    "for this repository" % (revision))
 
-        elif is_bstr(revision):
+        elif is_bstr:
             # get by branch/tag name
             _ref_revision = self._parsed_refs.get(revision)
-            _tags_shas = self.tags.values()
             if _ref_revision:  # and _ref_revision[1] in ['H', 'RH', 'T']:
                 return _ref_revision[0]
 
+            _tags_shas = self.tags.values()
             # maybe it's a tag ? we don't have them in self.revisions
-            elif revision in _tags_shas:
+            if revision in _tags_shas:
                 return _tags_shas[_tags_shas.index(revision)]
 
-            elif not pattern.match(revision) or revision not in self.revisions:
-                raise ChangesetDoesNotExistError("Revision %r does not exist "
-                    "for this repository %s" % (revision, self))
+            elif not SHA_PATTERN.match(revision) or revision not in self.revisions:
+                raise ChangesetDoesNotExistError("Revision %s does not exist "
+                    "for this repository" % (revision))
 
         # Ensure we return full id
-        if not pattern.match(str(revision)):
-            raise ChangesetDoesNotExistError("Given revision %r not recognized"
+        if not SHA_PATTERN.match(str(revision)):
+            raise ChangesetDoesNotExistError("Given revision %s not recognized"
                 % revision)
         return revision
 
@@ -281,6 +313,15 @@ class GitRepository(BaseRepository):
             url = ':///'.join(('file', url))
         return url
 
+    def get_hook_location(self):
+        """
+        returns absolute path to location where hooks are stored
+        """
+        loc = os.path.join(self.path, 'hooks')
+        if not self.bare:
+            loc = os.path.join(self.path, '.git', 'hooks')
+        return loc
+
     @LazyProperty
     def name(self):
         return os.path.basename(self.path)
@@ -329,6 +370,10 @@ class GitRepository(BaseRepository):
                      for x in self._parsed_refs.iteritems() if x[1][1] == 'H']
         return OrderedDict(sorted(_branches, key=sortkey, reverse=False))
 
+    @LazyProperty
+    def closed_branches(self):
+        return {}
+
     @LazyProperty
     def tags(self):
         return self._get_tags()
@@ -387,12 +432,21 @@ class GitRepository(BaseRepository):
         except OSError, e:
             raise RepositoryError(e.strerror)
 
+    @LazyProperty
+    def bookmarks(self):
+        """
+        Gets bookmarks for this repository
+        """
+        return {}
+
     @LazyProperty
     def _parsed_refs(self):
         return self._get_parsed_refs()
 
     def _get_parsed_refs(self):
-        refs = self._repo.get_refs()
+        # cache the property
+        _repo = self._repo
+        refs = _repo.get_refs()
         keys = [('refs/heads/', 'H'),
                 ('refs/remotes/origin/', 'RH'),
                 ('refs/tags/', 'T')]
@@ -401,6 +455,10 @@ class GitRepository(BaseRepository):
             for k, type_ in keys:
                 if ref.startswith(k):
                     _key = ref[len(k):]
+                    if type_ == 'T':
+                        obj = _repo.get_object(sha)
+                        if isinstance(obj, Tag):
+                            sha = _repo.get_object(sha).object[1]
                     _refs[_key] = [sha, type_]
                     break
         return _refs
@@ -455,6 +513,11 @@ class GitRepository(BaseRepository):
         if branch_name and branch_name not in self.branches:
             raise BranchDoesNotExistError("Branch '%s' not found" \
                                           % branch_name)
+        # actually we should check now if it's not an empty repo to not spaw
+        # subprocess commands
+        if self._empty:
+            raise EmptyRepositoryError("There are no changesets yet")
+
         # %H at format means (full) commit hash, initial hashes are retrieved
         # in ascending date order
         cmd_template = 'log --date-order --reverse --pretty=format:"%H"'
@@ -469,9 +532,10 @@ class GitRepository(BaseRepository):
             cmd_template += ' $branch_name'
             cmd_params['branch_name'] = branch_name
         else:
-            cmd_template += ' --all'
+            rev_filter = _git_path = settings.GIT_REV_FILTER
+            cmd_template += ' %s' % (rev_filter)
 
-        cmd = Template(cmd_template).safe_substitute(**cmd_params)
+        cmd = string.Template(cmd_template).safe_substitute(**cmd_params)
         revs = self.run_git_command(cmd)[0].splitlines()
         start_pos = 0
         end_pos = len(revs)
@@ -498,8 +562,7 @@ class GitRepository(BaseRepository):
         revs = revs[start_pos:end_pos]
         if reverse:
             revs = reversed(revs)
-        for rev in revs:
-            yield self.get_changeset(rev)
+        return CollectionGenerator(self, revs)
 
     def get_diff(self, rev1, rev2, path=None, ignore_whitespace=False,
                  context=3):
@@ -516,7 +579,7 @@ class GitRepository(BaseRepository):
         :param context: How many lines before/after changed lines should be
           shown. Defaults to ``3``.
         """
-        flags = ['-U%s' % context]
+        flags = ['-U%s' % context, '--full-index', '--binary', '-p', '-M', '--abbrev=40']
         if ignore_whitespace:
             flags.append('-w')
 
@@ -536,6 +599,7 @@ class GitRepository(BaseRepository):
 
         if path:
             cmd += ' -- "%s"' % path
+
         stdout, stderr = self.run_git_command(cmd)
         # If we used 'show' command, strip first few lines (until actual diff
         # starts)
@@ -582,9 +646,7 @@ class GitRepository(BaseRepository):
         Tries to pull changes from external location.
         """
         url = self._get_url(url)
-        cmd = ['pull']
-        cmd.append("--ff-only")
-        cmd.append(url)
+        cmd = ['pull', "--ff-only", url]
         cmd = ' '.join(cmd)
         # If error occurs run_git_command raises RepositoryError already
         self.run_git_command(cmd)
@@ -594,10 +656,13 @@ class GitRepository(BaseRepository):
         Tries to pull changes from external location.
         """
         url = self._get_url(url)
-        cmd = ['fetch']
-        cmd.append(url)
-        cmd = ' '.join(cmd)
-        # If error occurs run_git_command raises RepositoryError already
+        so, se = self.run_git_command('ls-remote -h %s' % url)
+        refs = []
+        for line in (x for x in so.splitlines()):
+            sha, ref = line.split('\t')
+            refs.append(ref)
+        refs = ' '.join(('+%s:%s' % (r, r) for r in refs))
+        cmd = '''fetch %s -- %s''' % (url, refs)
         self.run_git_command(cmd)
 
     @LazyProperty
