@@ -1,25 +1,46 @@
+# -*- coding: utf-8 -*-
+"""
+    vcs.backends.hg.repository
+    ~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+    Mercurial repository implementation.
+
+    :created_on: Apr 8, 2010
+    :copyright: (c) 2010-2011 by Marcin Kuzminski, Lukasz Balcerzak.
+"""
+
 import os
 import time
-import datetime
 import urllib
 import urllib2
+import logging
+import datetime
 
-from rhodecode.lib.vcs.backends.base import BaseRepository
-from .workdir import MercurialWorkdir
-from .changeset import MercurialChangeset
-from .inmemory import MercurialInMemoryChangeset
 
-from rhodecode.lib.vcs.exceptions import BranchDoesNotExistError, \
-    ChangesetDoesNotExistError, EmptyRepositoryError, RepositoryError, \
-    VCSError, TagAlreadyExistError, TagDoesNotExistError
-from rhodecode.lib.vcs.utils import author_email, author_name, date_fromtimestamp, \
-    makedate, safe_unicode
+from rhodecode.lib.vcs.backends.base import BaseRepository, CollectionGenerator
+from rhodecode.lib.vcs.conf import settings
+
+from rhodecode.lib.vcs.exceptions import (
+    BranchDoesNotExistError, ChangesetDoesNotExistError, EmptyRepositoryError,
+    RepositoryError, VCSError, TagAlreadyExistError, TagDoesNotExistError
+)
+from rhodecode.lib.vcs.utils import (
+    author_email, author_name, date_fromtimestamp, makedate, safe_unicode
+)
 from rhodecode.lib.vcs.utils.lazy import LazyProperty
 from rhodecode.lib.vcs.utils.ordered_dict import OrderedDict
 from rhodecode.lib.vcs.utils.paths import abspath
+from rhodecode.lib.vcs.utils.hgcompat import (
+    ui, nullid, match, patch, diffopts, clone, get_contact, pull,
+    localrepository, RepoLookupError, Abort, RepoError, hex, scmutil, hg_url,
+    httpbasicauthhandler, httpdigestauthhandler, peer
+)
 
-from rhodecode.lib.vcs.utils.hgcompat import ui, nullid, match, patch, diffopts, clone, \
-    get_contact, pull, localrepository, RepoLookupError, Abort, RepoError, hex
+from .changeset import MercurialChangeset
+from .inmemory import MercurialInMemoryChangeset
+from .workdir import MercurialWorkdir
+
+log = logging.getLogger(__name__)
 
 
 class MercurialRepository(BaseRepository):
@@ -57,7 +78,7 @@ class MercurialRepository(BaseRepository):
     @property
     def _empty(self):
         """
-        Checks if repository is empty without any changesets
+        Checks if repository is empty ie. without any changesets
         """
         # TODO: Following raises errors when using InMemoryChangeset...
         # return len(self._repo.changelog) == 0
@@ -79,12 +100,24 @@ class MercurialRepository(BaseRepository):
     def branches(self):
         return self._get_branches()
 
-    def _get_branches(self, closed=False):
+    @LazyProperty
+    def closed_branches(self):
+        return self._get_branches(normal=False, closed=True)
+
+    @LazyProperty
+    def allbranches(self):
         """
-        Get's branches for this repository
+        List all branches, including closed branches.
+        """
+        return self._get_branches(closed=True)
+
+    def _get_branches(self, normal=True, closed=False):
+        """
+        Gets branches for this repository
         Returns only not closed branches by default
 
         :param closed: return also closed branches for mercurial
+        :param normal: return also normal branches
         """
 
         if self._empty:
@@ -107,6 +140,8 @@ class MercurialRepository(BaseRepository):
                 else:
                     bt[bn] = tip
 
+            if not normal:
+                return bt_closed
             if closed:
                 bt.update(bt_closed)
             return bt
@@ -120,7 +155,7 @@ class MercurialRepository(BaseRepository):
     @LazyProperty
     def tags(self):
         """
-        Get's tags for this repository
+        Gets tags for this repository
         """
         return self._get_tags()
 
@@ -199,7 +234,7 @@ class MercurialRepository(BaseRepository):
     @LazyProperty
     def bookmarks(self):
         """
-        Get's bookmarks for this repository
+        Gets bookmarks for this repository
         """
         return self._get_bookmarks()
 
@@ -242,8 +277,11 @@ class MercurialRepository(BaseRepository):
         if rev1 != self.EMPTY_CHANGESET:
             self.get_changeset(rev1)
         self.get_changeset(rev2)
+        if path:
+            file_filter = match(self.path, '', [path])
+        else:
+            file_filter = None
 
-        file_filter = match(self.path, '', [path])
         return ''.join(patch.diff(self._repo, rev1, rev2, match=file_filter,
                           opts=diffopts(git=True,
                                         ignorews=ignore_whitespace,
@@ -261,12 +299,6 @@ class MercurialRepository(BaseRepository):
         is valid or True if it's a local path
         """
 
-        from mercurial.util import url as Url
-
-        # those authnadlers are patched for python 2.6.5 bug an
-        # infinit looping when given invalid resources
-        from mercurial.url import httpbasicauthhandler, httpdigestauthhandler
-
         # check first if it's not an local url
         if os.path.isdir(url) or url.startswith('file:'):
             return True
@@ -275,7 +307,7 @@ class MercurialRepository(BaseRepository):
             url = url[url.find('+') + 1:]
 
         handlers = []
-        test_uri, authinfo = Url(url).authinfo()
+        test_uri, authinfo = hg_url(url).authinfo()
 
         if authinfo:
             #create a password manager
@@ -379,7 +411,7 @@ class MercurialRepository(BaseRepository):
 
     def _get_revision(self, revision):
         """
-        Get's an ID revision given as str. This will always return a fill
+        Gets an ID revision given as str. This will always return a fill
         40 char revision number
 
         :param revision: str or int or None
@@ -394,9 +426,9 @@ class MercurialRepository(BaseRepository):
         try:
             revision = hex(self._repo.lookup(revision))
         except (IndexError, ValueError, RepoLookupError, TypeError):
-            raise ChangesetDoesNotExistError("Revision %r does not "
-                                    "exist for this repository %s" \
-                                    % (revision, self))
+            raise ChangesetDoesNotExistError("Revision %s does not "
+                                    "exist for this repository"
+                                    % (revision))
         return revision
 
     def _get_archives(self, archive_name='tip'):
@@ -419,6 +451,12 @@ class MercurialRepository(BaseRepository):
             url = "file:" + urllib.pathname2url(url)
         return url
 
+    def get_hook_location(self):
+        """
+        returns absolute path to location where hooks are stored
+        """
+        return os.path.join(self.path, '.hg', '.hgrc')
+
     def get_changeset(self, revision=None):
         """
         Returns ``MercurialChangeset`` object representing repository's
@@ -448,28 +486,35 @@ class MercurialRepository(BaseRepository):
         end_pos = self.revisions.index(end_raw_id) if end else None
 
         if None not in [start, end] and start_pos > end_pos:
-            raise RepositoryError("start revision '%s' cannot be "
+            raise RepositoryError("Start revision '%s' cannot be "
                                   "after end revision '%s'" % (start, end))
 
-        if branch_name and branch_name not in self.branches.keys():
-            raise BranchDoesNotExistError('Such branch %s does not exists for'
+        if branch_name and branch_name not in self.allbranches.keys():
+            raise BranchDoesNotExistError('Branch %s not found in'
                                   ' this repository' % branch_name)
         if end_pos is not None:
             end_pos += 1
+        #filter branches
+        filter_ = []
+        if branch_name:
+            filter_.append('branch("%s")' % (branch_name))
 
-        slice_ = reversed(self.revisions[start_pos:end_pos]) if reverse else \
-            self.revisions[start_pos:end_pos]
+        if start_date and not end_date:
+            filter_.append('date(">%s")' % start_date)
+        if end_date and not start_date:
+            filter_.append('date("<%s")' % end_date)
+        if start_date and end_date:
+            filter_.append('date(">%s") and date("<%s")' % (start_date, end_date))
+        if filter_:
+            revisions = scmutil.revrange(self._repo, filter_)
+        else:
+            revisions = self.revisions
 
-        for id_ in slice_:
-            cs = self.get_changeset(id_)
-            if branch_name and cs.branch != branch_name:
-                continue
-            if start_date and cs.date < start_date:
-                continue
-            if end_date and cs.date > end_date:
-                continue
+        revs = revisions[start_pos:end_pos]
+        if reverse:
+            revs = reversed(revs)
 
-            yield cs
+        return CollectionGenerator(self, revs)
 
     def pull(self, url):
         """
@@ -477,7 +522,8 @@ class MercurialRepository(BaseRepository):
         """
         url = self._get_url(url)
         try:
-            pull(self.baseui, self._repo, url)
+            other = peer(self._repo, {}, url)
+            self._repo.pull(other, heads=None, force=None)
         except Abort, err:
             # Propagate error but with vcs's type
             raise RepositoryError(str(err))
@@ -489,7 +535,7 @@ class MercurialRepository(BaseRepository):
         """
         return MercurialWorkdir(self)
 
-    def get_config_value(self, section, name, config_file=None):
+    def get_config_value(self, section, name=None, config_file=None):
         """
         Returns configuration value for a given [``section``] and ``name``.
 
