@@ -1,5 +1,6 @@
 import os
 import tempfile
+from rhodecode.lib.vcs.utils import aslist
 from rhodecode.lib.vcs.utils.paths import get_user_home
 
 abspath = lambda * p: os.path.abspath(os.path.join(*p))
@@ -15,9 +16,17 @@ VCSRC_PATH = VCSRC_PATH or abspath(HOME_, '.vcsrc')
 if os.path.isdir(VCSRC_PATH):
     VCSRC_PATH = os.path.join(VCSRC_PATH, '__init__.py')
 
+# list of default encoding used in safe_unicode/safe_str methods
+DEFAULT_ENCODINGS = aslist('utf8')
+
+# path to git executable runned by run_git_command function
+GIT_EXECUTABLE_PATH = 'git'
+# can be also --branches --tags
+GIT_REV_FILTER = '--all'
+
 BACKENDS = {
-    'hg': 'vcs.backends.hg.MercurialRepository',
-    'git': 'vcs.backends.git.GitRepository',
+    'hg': 'rhodecode.lib.vcs.backends.hg.MercurialRepository',
+    'git': 'rhodecode.lib.vcs.backends.git.GitRepository',
 }
 
 ARCHIVE_SPECS = {
@@ -26,8 +35,3 @@ ARCHIVE_SPECS = {
     'tgz': ('application/x-gzip', '.tar.gz'),
     'zip': ('application/zip', '.zip'),
 }
-
-BACKENDS = {
-    'hg': 'rhodecode.lib.vcs.backends.hg.MercurialRepository',
-    'git': 'rhodecode.lib.vcs.backends.git.GitRepository',
-}
