@@ -30,7 +30,7 @@ import traceback
 import logging
 import cStringIO
 import pkg_resources
-from os.path import dirname as dn, join as jn
+from os.path import join as jn
 
 from sqlalchemy import func
 from pylons.i18n.translation import _
@@ -44,13 +44,17 @@ from rhodecode.lib.vcs.backends.base import EmptyChangeset
 
 from rhodecode import BACKENDS
 from rhodecode.lib import helpers as h
-from rhodecode.lib.utils2 import safe_str, safe_unicode
-from rhodecode.lib.auth import HasRepoPermissionAny, HasReposGroupPermissionAny
-from rhodecode.lib.utils import get_repos as get_filesystem_repos, make_ui, \
-    action_logger, REMOVED_REPO_PAT
+from rhodecode.lib.utils2 import safe_str, safe_unicode, get_server_url,\
+    _set_extras
+from rhodecode.lib.auth import HasRepoPermissionAny, HasReposGroupPermissionAny,\
+    HasUserGroupPermissionAny
+from rhodecode.lib.utils import get_filesystem_repos, make_ui, \
+    action_logger
 from rhodecode.model import BaseModel
 from rhodecode.model.db import Repository, RhodeCodeUi, CacheInvalidation, \
     UserFollowing, UserLog, User, RepoGroup, PullRequest
+from rhodecode.lib.hooks import log_push_action
+from rhodecode.lib.exceptions import NonRelativePathError, IMCCommitError
 
 log = logging.getLogger(__name__)
 
@@ -77,11 +81,15 @@ class CachedRepoList(object):
     super fast
     """
 
-    def __init__(self, db_repo_list, repos_path, order_by=None):
+    def __init__(self, db_repo_list, repos_path, order_by=None, perm_set=None):
         self.db_repo_list = db_repo_list
         self.repos_path = repos_path
         self.order_by = order_by
         self.reversed = (order_by or '').startswith('-')
+        if not perm_set:
+            perm_set = ['repository.read', 'repository.write',
+                        'repository.admin']
+        self.perm_set = perm_set
 
     def __len__(self):
         return len(self.db_repo_list)
@@ -90,31 +98,32 @@ class CachedRepoList(object):
         return '<%s (%s)>' % (self.__class__.__name__, self.__len__())
 
     def __iter__(self):
-        # pre-propagated cache_map to save executing select statements
+        # pre-propagated valid_cache_keys to save executing select statements
         # for each repo
-        cache_map = CacheInvalidation.get_cache_map()
+        valid_cache_keys = CacheInvalidation.get_valid_cache_keys()
 
         for dbr in self.db_repo_list:
-            scmr = dbr.scm_instance_cached(cache_map)
+            scmr = dbr.scm_instance_cached(valid_cache_keys)
             # check permission at this level
             if not HasRepoPermissionAny(
-                'repository.read', 'repository.write', 'repository.admin'
-            )(dbr.repo_name, 'get repo check'):
+                *self.perm_set)(dbr.repo_name, 'get repo check'):
                 continue
 
-            if scmr is None:
+            try:
+                last_change = scmr.last_change
+                tip = h.get_changeset_safe(scmr, 'tip')
+            except Exception:
                 log.error(
                     '%s this repository is present in database but it '
-                    'cannot be created as an scm instance' % dbr.repo_name
+                    'cannot be created as an scm instance, org_exc:%s'
+                    % (dbr.repo_name, traceback.format_exc())
                 )
                 continue
 
-            last_change = scmr.last_change
-            tip = h.get_changeset_safe(scmr, 'tip')
-
             tmp_d = {}
             tmp_d['name'] = dbr.repo_name
             tmp_d['name_sort'] = tmp_d['name'].lower()
+            tmp_d['raw_name'] = tmp_d['name'].lower()
             tmp_d['description'] = dbr.description
             tmp_d['description_sort'] = tmp_d['description'].lower()
             tmp_d['last_change'] = last_change
@@ -142,13 +151,13 @@ class SimpleCachedRepoList(CachedRepoList):
         for dbr in self.db_repo_list:
             # check permission at this level
             if not HasRepoPermissionAny(
-                'repository.read', 'repository.write', 'repository.admin'
-            )(dbr.repo_name, 'get repo check'):
+                *self.perm_set)(dbr.repo_name, 'get repo check'):
                 continue
 
             tmp_d = {}
             tmp_d['name'] = dbr.repo_name
             tmp_d['name_sort'] = tmp_d['name'].lower()
+            tmp_d['raw_name'] = tmp_d['name'].lower()
             tmp_d['description'] = dbr.description
             tmp_d['description_sort'] = tmp_d['description'].lower()
             tmp_d['dbrepo'] = dbr.get_dict()
@@ -156,26 +165,69 @@ class SimpleCachedRepoList(CachedRepoList):
             yield tmp_d
 
 
-class GroupList(object):
+class _PermCheckIterator(object):
+    def __init__(self, obj_list, obj_attr, perm_set, perm_checker):
+        """
+        Creates iterator from given list of objects, additionally
+        checking permission for them from perm_set var
 
-    def __init__(self, db_repo_group_list):
-        self.db_repo_group_list = db_repo_group_list
+        :param obj_list: list of db objects
+        :param obj_attr: attribute of object to pass into perm_checker
+        :param perm_set: list of permissions to check
+        :param perm_checker: callable to check permissions against
+        """
+        self.obj_list = obj_list
+        self.obj_attr = obj_attr
+        self.perm_set = perm_set
+        self.perm_checker = perm_checker
 
     def __len__(self):
-        return len(self.db_repo_group_list)
+        return len(self.obj_list)
 
     def __repr__(self):
         return '<%s (%s)>' % (self.__class__.__name__, self.__len__())
 
     def __iter__(self):
-        for dbgr in self.db_repo_group_list:
+        for db_obj in self.obj_list:
             # check permission at this level
-            if not HasReposGroupPermissionAny(
-                'group.read', 'group.write', 'group.admin'
-            )(dbgr.group_name, 'get group repo check'):
+            name = getattr(db_obj, self.obj_attr, None)
+            if not self.perm_checker(*self.perm_set)(name, self.__class__.__name__):
                 continue
 
-            yield dbgr
+            yield db_obj
+
+
+class RepoList(_PermCheckIterator):
+
+    def __init__(self, db_repo_list, perm_set=None):
+        if not perm_set:
+            perm_set = ['repository.read', 'repository.write', 'repository.admin']
+
+        super(RepoList, self).__init__(obj_list=db_repo_list,
+                    obj_attr='repo_name', perm_set=perm_set,
+                    perm_checker=HasRepoPermissionAny)
+
+
+class RepoGroupList(_PermCheckIterator):
+
+    def __init__(self, db_repo_group_list, perm_set=None):
+        if not perm_set:
+            perm_set = ['group.read', 'group.write', 'group.admin']
+
+        super(RepoGroupList, self).__init__(obj_list=db_repo_group_list,
+                    obj_attr='group_name', perm_set=perm_set,
+                    perm_checker=HasReposGroupPermissionAny)
+
+
+class UserGroupList(_PermCheckIterator):
+
+    def __init__(self, db_user_group_list, perm_set=None):
+        if not perm_set:
+            perm_set = ['usergroup.read', 'usergroup.write', 'usergroup.admin']
+
+        super(UserGroupList, self).__init__(obj_list=db_user_group_list,
+                    obj_attr='users_group_name', perm_set=perm_set,
+                    perm_checker=HasUserGroupPermissionAny)
 
 
 class ScmModel(BaseModel):
@@ -198,7 +250,7 @@ class ScmModel(BaseModel):
     @LazyProperty
     def repos_path(self):
         """
-        Get's the repositories root path from database
+        Gets the repositories root path from database
         """
 
         q = self.sa.query(RhodeCodeUi).filter(RhodeCodeUi.ui_key == '/').one()
@@ -222,13 +274,9 @@ class ScmModel(BaseModel):
         repos = {}
 
         for name, path in get_filesystem_repos(repos_path, recursive=True):
-            # skip removed repos
-            if REMOVED_REPO_PAT.match(name):
-                continue
-
             # name need to be decomposed and put back together using the /
             # since this is internal storage separator for rhodecode
-            name = Repository.url_sep().join(name.split(os.sep))
+            name = Repository.normalize_repo_name(name)
 
             try:
                 if name in repos:
@@ -245,7 +293,7 @@ class ScmModel(BaseModel):
                         repos[name] = klass(path[1])
             except OSError:
                 continue
-
+        log.debug('found %s paths with repositories' % (len(repos)))
         return repos
 
     def get_repos(self, all_repos=None, sort_key=None, simple=False):
@@ -278,18 +326,18 @@ class ScmModel(BaseModel):
         if all_groups is None:
             all_groups = RepoGroup.query()\
                 .filter(RepoGroup.group_parent_id == None).all()
-        group_iter = GroupList(all_groups)
-
-        return group_iter
+        return [x for x in RepoGroupList(all_groups)]
 
     def mark_for_invalidation(self, repo_name):
         """
-        Puts cache invalidation task into db for
-        further global cache invalidation
+        Mark caches of this repo invalid in the database.
 
-        :param repo_name: this repo that should invalidation take place
+        :param repo_name: the repo for which caches should be marked invalid
         """
         CacheInvalidation.set_invalidate(repo_name)
+        repo = Repository.get_by_repo_name(repo_name)
+        if repo:
+            repo.update_changeset_cache()
 
     def toggle_following_repo(self, follow_repo_id, user_id):
 
@@ -304,7 +352,7 @@ class ScmModel(BaseModel):
                               'stopped_following_repo',
                               RepoTemp(follow_repo_id))
                 return
-            except:
+            except Exception:
                 log.error(traceback.format_exc())
                 raise
 
@@ -317,7 +365,7 @@ class ScmModel(BaseModel):
             action_logger(UserTemp(user_id),
                           'started_following_repo',
                           RepoTemp(follow_repo_id))
-        except:
+        except Exception:
             log.error(traceback.format_exc())
             raise
 
@@ -330,7 +378,7 @@ class ScmModel(BaseModel):
             try:
                 self.sa.delete(f)
                 return
-            except:
+            except Exception:
                 log.error(traceback.format_exc())
                 raise
 
@@ -339,7 +387,7 @@ class ScmModel(BaseModel):
             f.user_id = user_id
             f.follows_user_id = follow_user_id
             self.sa.add(f)
-        except:
+        except Exception:
             log.error(traceback.format_exc())
             raise
 
@@ -376,7 +424,8 @@ class ScmModel(BaseModel):
     def get_pull_requests(self, repo):
         repo = self._get_repo(repo)
         return self.sa.query(PullRequest)\
-                .filter(PullRequest.other_repo == repo).count()
+                .filter(PullRequest.other_repo == repo)\
+                .filter(PullRequest.status != PullRequest.STATUS_CLOSED).count()
 
     def mark_as_fork(self, repo, fork, user):
         repo = self.__get_repo(repo)
@@ -387,6 +436,66 @@ class ScmModel(BaseModel):
         self.sa.add(repo)
         return repo
 
+    def _handle_rc_scm_extras(self, username, repo_name, repo_alias,
+                              action=None):
+        from rhodecode import CONFIG
+        from rhodecode.lib.base import _get_ip_addr
+        try:
+            from pylons import request
+            environ = request.environ
+        except TypeError:
+            # we might use this outside of request context, let's fake the
+            # environ data
+            from webob import Request
+            environ = Request.blank('').environ
+        extras = {
+            'ip': _get_ip_addr(environ),
+            'username': username,
+            'action': action or 'push_local',
+            'repository': repo_name,
+            'scm': repo_alias,
+            'config': CONFIG['__file__'],
+            'server_url': get_server_url(environ),
+            'make_lock': None,
+            'locked_by': [None, None]
+        }
+        _set_extras(extras)
+
+    def _handle_push(self, repo, username, action, repo_name, revisions):
+        """
+        Triggers push action hooks
+
+        :param repo: SCM repo
+        :param username: username who pushes
+        :param action: push/push_loca/push_remote
+        :param repo_name: name of repo
+        :param revisions: list of revisions that we pushed
+        """
+        self._handle_rc_scm_extras(username, repo_name, repo_alias=repo.alias)
+        _scm_repo = repo._repo
+        # trigger push hook
+        if repo.alias == 'hg':
+            log_push_action(_scm_repo.ui, _scm_repo, node=revisions[0])
+        elif repo.alias == 'git':
+            log_push_action(None, _scm_repo, _git_revs=revisions)
+
+    def _get_IMC_module(self, scm_type):
+        """
+        Returns InMemoryCommit class based on scm_type
+
+        :param scm_type:
+        """
+        if scm_type == 'hg':
+            from rhodecode.lib.vcs.backends.hg import MercurialInMemoryChangeset
+            return MercurialInMemoryChangeset
+
+        if scm_type == 'git':
+            from rhodecode.lib.vcs.backends.git import GitInMemoryChangeset
+            return GitInMemoryChangeset
+
+        raise Exception('Invalid scm_type, must be one of hg,git got %s'
+                        % (scm_type,))
+
     def pull_changes(self, repo, username):
         dbrepo = self.__get_repo(repo)
         clone_uri = dbrepo.clone_uri
@@ -394,22 +503,25 @@ class ScmModel(BaseModel):
             raise Exception("This repository doesn't have a clone uri")
 
         repo = dbrepo.scm_instance
+        repo_name = dbrepo.repo_name
         try:
-            extras = {
-                'ip': '',
-                'username': username,
-                'action': 'push_remote',
-                'repository': dbrepo.repo_name,
-                'scm': repo.alias,
-            }
-            Repository.inject_ui(repo, extras=extras)
-
             if repo.alias == 'git':
                 repo.fetch(clone_uri)
+                # git doesn't really have something like post-fetch action
+                # we fake that now. #TODO: extract fetched revisions somehow
+                # here
+                self._handle_push(repo,
+                                  username=username,
+                                  action='push_remote',
+                                  repo_name=repo_name,
+                                  revisions=[])
             else:
+                self._handle_rc_scm_extras(username, dbrepo.repo_name,
+                                           repo.alias, action='push_remote')
                 repo.pull(clone_uri)
-            self.mark_for_invalidation(dbrepo.repo_name)
-        except:
+
+            self.mark_for_invalidation(repo_name)
+        except Exception:
             log.error(traceback.format_exc())
             raise
 
@@ -421,13 +533,8 @@ class ScmModel(BaseModel):
         :param repo: SCM instance
 
         """
-
-        if repo.alias == 'hg':
-            from rhodecode.lib.vcs.backends.hg import \
-                MercurialInMemoryChangeset as IMC
-        elif repo.alias == 'git':
-            from rhodecode.lib.vcs.backends.git import \
-                GitInMemoryChangeset as IMC
+        user = self._get_user(user)
+        IMC = self._get_IMC_module(repo.alias)
 
         # decoding here will force that we have proper encoded values
         # in any other case this will throw exceptions and deny commit
@@ -437,54 +544,94 @@ class ScmModel(BaseModel):
         # proper backend should then translate that into required type
         message = safe_unicode(message)
         author = safe_unicode(author)
-        m = IMC(repo)
-        m.change(FileNode(path, content))
-        tip = m.commit(message=message,
-                       author=author,
-                       parents=[cs], branch=cs.branch)
-
-        action = 'push_local:%s' % tip.raw_id
-        action_logger(user, action, repo_name)
-        self.mark_for_invalidation(repo_name)
+        imc = IMC(repo)
+        imc.change(FileNode(path, content, mode=cs.get_file_mode(f_path)))
+        try:
+            tip = imc.commit(message=message, author=author,
+                             parents=[cs], branch=cs.branch)
+        except Exception, e:
+            log.error(traceback.format_exc())
+            raise IMCCommitError(str(e))
+        finally:
+            # always clear caches, if commit fails we want fresh object also
+            self.mark_for_invalidation(repo_name)
+        self._handle_push(repo,
+                          username=user.username,
+                          action='push_local',
+                          repo_name=repo_name,
+                          revisions=[tip.raw_id])
         return tip
 
-    def create_node(self, repo, repo_name, cs, user, author, message, content,
-                      f_path):
-        if repo.alias == 'hg':
-            from rhodecode.lib.vcs.backends.hg import MercurialInMemoryChangeset as IMC
-        elif repo.alias == 'git':
-            from rhodecode.lib.vcs.backends.git import GitInMemoryChangeset as IMC
-        # decoding here will force that we have proper encoded values
-        # in any other case this will throw exceptions and deny commit
+    def create_nodes(self, user, repo, message, nodes, parent_cs=None,
+                     author=None, trigger_push_hook=True):
+        """
+        Commits given multiple nodes into repo
 
-        if isinstance(content, (basestring,)):
-            content = safe_str(content)
-        elif isinstance(content, (file, cStringIO.OutputType,)):
-            content = content.read()
-        else:
-            raise Exception('Content is of unrecognized type %s' % (
-                type(content)
-            ))
+        :param user: RhodeCode User object or user_id, the commiter
+        :param repo: RhodeCode Repository object
+        :param message: commit message
+        :param nodes: mapping {filename:{'content':content},...}
+        :param parent_cs: parent changeset, can be empty than it's initial commit
+        :param author: author of commit, cna be different that commiter only for git
+        :param trigger_push_hook: trigger push hooks
+
+        :returns: new commited changeset
+        """
+
+        user = self._get_user(user)
+        scm_instance = repo.scm_instance_no_cache()
+
+        processed_nodes = []
+        for f_path in nodes:
+            if f_path.startswith('/') or f_path.startswith('.') or '../' in f_path:
+                raise NonRelativePathError('%s is not an relative path' % f_path)
+            if f_path:
+                f_path = os.path.normpath(f_path)
+            content = nodes[f_path]['content']
+            f_path = safe_str(f_path)
+            # decoding here will force that we have proper encoded values
+            # in any other case this will throw exceptions and deny commit
+            if isinstance(content, (basestring,)):
+                content = safe_str(content)
+            elif isinstance(content, (file, cStringIO.OutputType,)):
+                content = content.read()
+            else:
+                raise Exception('Content is of unrecognized type %s' % (
+                    type(content)
+                ))
+            processed_nodes.append((f_path, content))
 
         message = safe_unicode(message)
-        author = safe_unicode(author)
-        path = safe_str(f_path)
-        m = IMC(repo)
+        commiter = user.full_contact
+        author = safe_unicode(author) if author else commiter
 
-        if isinstance(cs, EmptyChangeset):
+        IMC = self._get_IMC_module(scm_instance.alias)
+        imc = IMC(scm_instance)
+
+        if not parent_cs:
+            parent_cs = EmptyChangeset(alias=scm_instance.alias)
+
+        if isinstance(parent_cs, EmptyChangeset):
             # EmptyChangeset means we we're editing empty repository
             parents = None
         else:
-            parents = [cs]
+            parents = [parent_cs]
+        # add multiple nodes
+        for path, content in processed_nodes:
+            imc.add(FileNode(path, content=content))
 
-        m.add(FileNode(path, content=content))
-        tip = m.commit(message=message,
-                       author=author,
-                       parents=parents, branch=cs.branch)
+        tip = imc.commit(message=message,
+                         author=author,
+                         parents=parents,
+                         branch=parent_cs.branch)
 
-        action = 'push_local:%s' % tip.raw_id
-        action_logger(user, action, repo_name)
-        self.mark_for_invalidation(repo_name)
+        self.mark_for_invalidation(repo.repo_name)
+        if trigger_push_hook:
+            self._handle_push(scm_instance,
+                              username=user.username,
+                              action='push_local',
+                              repo_name=repo.repo_name,
+                              revisions=[tip.raw_id])
         return tip
 
     def get_nodes(self, repo_name, revision, root_path='/', flat=True):
@@ -526,7 +673,6 @@ class ScmModel(BaseModel):
         grouped by type
 
         :param repo:
-        :type repo:
         """
 
         hist_l = []
@@ -585,7 +731,6 @@ class ScmModel(BaseModel):
             if os.path.exists(_hook_file):
                 # let's take a look at this hook, maybe it's rhodecode ?
                 log.debug('hook exists, checking if it is from rhodecode')
-                _HOOK_VER_PAT = re.compile(r'^RC_HOOK_VER')
                 with open(_hook_file, 'rb') as f:
                     data = f.read()
                     matches = re.compile(r'(?:%s)\s*=\s*(.*)'
@@ -595,17 +740,20 @@ class ScmModel(BaseModel):
                             ver = matches.groups()[0]
                             log.debug('got %s it is rhodecode' % (ver))
                             _rhodecode_hook = True
-                        except:
+                        except Exception:
                             log.error(traceback.format_exc())
             else:
                 # there is no hook in this dir, so we want to create one
                 _rhodecode_hook = True
 
             if _rhodecode_hook or force_create:
-                log.debug('writing %s hook file !' % h_type)
-                with open(_hook_file, 'wb') as f:
-                    tmpl = tmpl.replace('_TMPL_', rhodecode.__version__)
-                    f.write(tmpl)
-                os.chmod(_hook_file, 0755)
+                log.debug('writing %s hook file !' % (h_type,))
+                try:
+                    with open(_hook_file, 'wb') as f:
+                        tmpl = tmpl.replace('_TMPL_', rhodecode.__version__)
+                        f.write(tmpl)
+                    os.chmod(_hook_file, 0755)
+                except IOError, e:
+                    log.error('error writing %s: %s' % (_hook_file, e))
             else:
                 log.debug('skipping writing hook file')
