@@ -3,6 +3,7 @@ from rhodecode.model.db import Notification, User
 
 from rhodecode.model.user import UserModel
 from rhodecode.model.notification import NotificationModel
+from rhodecode.model.meta import Session
 
 
 class TestNotificationsController(TestController):
@@ -10,8 +11,8 @@ class TestNotificationsController(TestController):
     def tearDown(self):
         for n in Notification.query().all():
             inst = Notification.get(n.notification_id)
-            self.Session().delete(inst)
-        self.Session().commit()
+            Session().delete(inst)
+        Session().commit()
 
     def test_index(self):
         self.log_user()
@@ -22,17 +23,16 @@ class TestNotificationsController(TestController):
         u1 = u1.user_id
 
         response = self.app.get(url('notifications'))
-        self.assertTrue('''<div class="table">No notifications here yet</div>'''
-                        in response.body)
+        response.mustcontain('<div class="table">No notifications here yet</div>')
 
         cur_user = self._get_logged_user()
 
         NotificationModel().create(created_by=u1, subject=u'test_notification_1',
                                    body=u'notification_1',
                                    recipients=[cur_user])
-        self.Session().commit()
+        Session().commit()
         response = self.app.get(url('notifications'))
-        self.assertTrue(u'test_notification_1' in response.body)
+        response.mustcontain(u'test_notification_1')
 
 #    def test_index_as_xml(self):
 #        response = self.app.get(url('formatted_notifications', format='xml'))
@@ -68,7 +68,7 @@ class TestNotificationsController(TestController):
                                                   subject=u'test',
                                                   body=u'hi there',
                                                   recipients=[cur_user, u1, u2])
-        self.Session().commit()
+        Session().commit()
         u1 = User.get(u1.user_id)
         u2 = User.get(u2.user_id)
 
@@ -82,6 +82,7 @@ class TestNotificationsController(TestController):
         response = self.app.delete(url('notification',
                                        notification_id=
                                        notification.notification_id))
+        self.assertEqual(response.body, 'ok')
 
         cur_user = User.get(cur_usr_id)
         self.assertEqual(cur_user.notifications, [])
