@@ -4,13 +4,13 @@ import os
 from rhodecode.lib.vcs.backends.hg import MercurialRepository, MercurialChangeset
 from rhodecode.lib.vcs.exceptions import RepositoryError, VCSError, NodeDoesNotExistError
 from rhodecode.lib.vcs.nodes import NodeKind, NodeState
-from conf import PACKAGE_DIR, TEST_HG_REPO, TEST_HG_REPO_CLONE, \
+from rhodecode.tests.vcs.conf import PACKAGE_DIR, TEST_HG_REPO, TEST_HG_REPO_CLONE, \
     TEST_HG_REPO_PULL
 from rhodecode.lib.vcs.utils.compat import unittest
 
 
 # Use only clean mercurial's ui
-import mercurial.scmutil
+from rhodecode.lib.vcs.utils.hgcompat import mercurial
 mercurial.scmutil.rcpath()
 if mercurial.scmutil._rcpath:
     mercurial.scmutil._rcpath = mercurial.scmutil._rcpath[:1]
@@ -156,9 +156,10 @@ class MercurialRepositoryTest(unittest.TestCase):
 
         #active branches
         self.assertTrue('default' in self.repo.branches)
-        self.assertTrue('git' in self.repo.branches)
+        self.assertTrue('stable' in self.repo.branches)
 
         # closed
+        self.assertTrue('git' in self.repo._get_branches(closed=True))
         self.assertTrue('web' in self.repo._get_branches(closed=True))
 
         for name, id in self.repo.branches.items():
@@ -392,13 +393,13 @@ class MercurialChangesetTest(unittest.TestCase):
                                  55: {'lines_no': 3,
                                      'changesets': [7, 7, 7]}}}
 
-
         for fname, revision_dict in files.items():
             for rev, data in revision_dict.items():
                 cs = self.repo.get_changeset(rev)
-                ann = cs.get_file_annotate(fname)
-
-                l1 = [x[1].revision for x in ann]
+                l1_1 = [x[1] for x in cs.get_file_annotate(fname)]
+                l1_2 = [x[2]().raw_id for x in cs.get_file_annotate(fname)]
+                self.assertEqual(l1_1, l1_2)
+                l1 = l1_2 = [x[2]().revision for x in cs.get_file_annotate(fname)]
                 l2 = files[fname][rev]['changesets']
                 self.assertTrue(l1 == l2 , "The lists of revision for %s@rev%s"
                                 "from annotation list should match each other,"
